# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimList,
    HprimMultiple,
    HprimString,
    HprimDate,
    HprimForeign,
)


class EvolisSegmentH(HeaderSegment):
    sender_id = HprimString(5)
    receiver_id = HprimString(10)
    version = HprimString(13)
    date = HprimDate(14)
    patients = HprimForeign("P", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class EvolisSegmentP(Segment):
    practice_pid = HprimString(3)
    # contient le numéro d'échantillon
    laboratory_pid = HprimString(4)
    patient_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)
    attending_physician = HprimString(14)
    orders = HprimForeign("O", "patient")


class EvolisSegmentO(Segment):
    specimen_id = HprimString(3)
    instrument_specimen_id = HprimString(4)
    tests = HprimList(5, basefield=HprimMultiple(
        0, code=HprimString(4)))
    collection_date = HprimDate(8)
    specimen_descriptor = HprimString(16)
    report_type = HprimString(26)
    results = HprimForeign("R", "order")


class EvolisSegmentR(Segment):
    test_name = HprimString((3, 4))
    test_code = HprimString((3, 5))
    data = HprimString(4)
    units = HprimString(5)
    reference_range = HprimString(6)
    flags = HprimList(7)
    result_status = HprimString(9)
    date = HprimDate(14)


class EvolisSegmentC(Segment):
    comment_source = HprimString(3)
    reactif = HprimMultiple(
        4,
        test_reactif=HprimString(1),
        lot_reactif=HprimString(2),
        date_peremption=HprimDate(3, fmt_date="%d/%m/%Y"),
    )
    comment_type = HprimString(5)


class EvolisSegmentL(Segment):
    terminaison_code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "evolis",
        "label": "Evolis",
    }
    SEGMENT_CLASS_MAP = {
        "H": EvolisSegmentH,
        "P": EvolisSegmentP,
        "O": EvolisSegmentO,
        "R": EvolisSegmentR,
        "C": EvolisSegmentC,
        "L": EvolisSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.Option(
            "evolis_reference_qc",
            label="Références de QC sépararés par des \";\" (hors indication du niveau) dans le champ O.3",
            default="NC;PC;CO"
        )
    ]

    def __init__(self, **kwargs):
        self.reference_qc = kwargs.pop("evolis_reference_qc", "NC;PC;CO")
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = EvolisSegmentH(type_segment="H", **kwargs)
        message.end = EvolisSegmentL(type_segment="L")
        return message

    def get_message_type(self, message):
        return self.ORDER

    def encode_worklist(self, worklist, context):
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: None},
        ):
            physician = [p for p in patient.physicians if p.type.lower() == 'prescriber']
            seg_p = msg.patients.add(
                practice_pid=patient.patient_id,
                patient_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
                attending_physician=physician[0].code if physician else "",
            )
            for sample, action in patient.samples:
                seg_o = seg_p.orders.add(
                    specimen_id=sample.sample_id,
                    collection_date=sample.collection_date,
                    # il faut envoyer S pour Sample
                    specimen_descriptor=sample.specimen,
                    report_type="X",
                )
                seg_p.laboratory_pid = sample.sample_id
                for order in sample.orders:
                    t = seg_o.tests.add()
                    t.code = order.code

        if msg.patients:
            return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                if (
                    not seg_p.practice_pid and
                    not seg_p.laboratory_pid and
                    seg_o.instrument_specimen_id.startswith(tuple(self.reference_qc.split(";")))
                ):
                    results = worklist.get_or_create(
                        sample_id=seg_o.instrument_specimen_id,
                        control_lot=seg_o.instrument_specimen_id,
                        type="control"
                    ).results
                else:
                    results = worklist.get_or_create(
                        sample_id=seg_p.laboratory_pid.strip(),
                        type="patient"
                    ).results
                for seg_r in seg_o.results:
                    lots_reactifs = []
                    for seg_c in seg_r.commentaires:
                        lots_reactifs.append({
                            "name": seg_c.reactif.test_reactif,
                            "lot": seg_c.reactif.lot_reactif,
                            "expiration": seg_c.reactif.date_peremption,
                            "type": "reagent",
                        })
                    results.add(
                        code=seg_r.test_code,
                        value=seg_r.data,
                        flags=seg_r.flags,
                        date=seg_r.date or message.date,
                        lots=lots_reactifs,
                    )
        return worklist
